#!/usr/bin/env python3

import matplotlib.pyplot as plt
from matplotlib import style
from matplotlib.dates import DateFormatter
import matplotlib.dates as mdates
from matplotlib.ticker import (MultipleLocator, AutoMinorLocator)
import matplotlib.ticker as ticker
import sys

import pandas as pd
import numpy as np
import sqlite3
import datetime
import random
from dateutil import parser
from dateutil.relativedelta import relativedelta

from chart_functions import *

MENU=True        # When False skip the menu and use default variables.
RANGE_STATS=True # Status relative to range instead of all time. When false 'All' is in legend.
SHOW_CHART=True
STATS=True       # Show dashed lines for poor, average, and great.
#Database or CSV column names
NAME='ename'
DATE='mydate'
END_DATE = pd.to_datetime(datetime.date.today()) #for pandas to use the date
DEBUG=False # Show info in yellow.
CUSTOM=False #Place holder for custom date
TIME={'years':2}
ALIAS='bench'

connect = sqlite3.connect("data.db")
df = pd.read_sql_query("Select * from exerciseLogs;", connect,parse_dates=DATE,)

# Add your alias and exercise name here. These are queries.
lifts={
		'bench':'Barbell Bench Press',
		'incline':'Barbell Standing Military Press',
		'sh':'Barbell Standing Military Press',
		'squat':'Barbell Squat',
		'lp':'Leg Press',
		'dl':'Barbell Deadlift',
		'dbp':'Dumbbell Bench Press',
		}
times = {
		'w':{'weeks':1},
		'y':{'years':1},
		'm':{'months':1},
		'a':{'years':20},
		'tm':{'months':3},
		'tw':{'weeks':3},
		'ty':{'years':3},
		'sm':{'months':6},
		'sw':{'weeks':6},
		'ty':{'years':3},
		'ty':{'years':3},
		'r':'range',
		'e':'exit',
		'q':'quit',
		'h': 'print lift alias'
		}
def main():
	start_date=''; end_date=''
	if MENU:
		returned_obj=''
		while True:
			returned_obj = MENU_DIALOG(ALIAS,TIME,lifts,times) #I pass them, not too familiar with now to modify globals
			if isinstance(returned_obj,dict): # returned_obj = {'bench': {'years': 1}}
				RESULTS = prep_data(returned_obj)
				print("return:",returned_obj,"MOD:==", RESULTS, "\n===")
			elif isinstance(returned_obj,list):
				#Todo 3: learn more about except
				try:
					start_date=pd.to_datetime(returned_obj[0])
					end_date=pd.to_datetime(returned_obj[1])
				except:
					print(RED+'In pandas converting to_datetime failed. Try entering the correct date format jan 23 19.'+NO)
					continue
				RESULTS=prep_data(returned_obj)
			else:
				print(RED+"Something went wrong in the loop"+NO)
				exit()
			print("\n",RESULTS.head(50)) # Show only 50 items.
			if SHOW_CHART: CHART(RESULTS)
	else:
		print('Menu is off, using default variables.')
		returned_obj = MENU_DIALOG(ALIAS,TIME,lifts,times)
		RESULTS = prep_data(returned_obj)

# PANDAS INITIALIZATION
def prep_data(*args):
	""" Valid arguments are:
	For range (['jan 1 2016', 'jan 1 2017', 'bench'],)
	For args  ({'bench': {'years': 1}},)"""
	print("args: ", args)
	global df; global END_DATE;

	dt=relativedelta(**TIME)
	dt_today=datetime.date.today()

	if isinstance(args[0],dict):
		for myalias in args[0]: #Assuming we are not passing more than 1 key (alias) and 1 object (kwargs)
			ALIAS=myalias #i.e bench
			myKWargs=args[0][myalias] #i.e {'months':1}
			START_DATE=pd.to_datetime(dt_today-relativedelta(**myKWargs)) #For pandas to work with the date
	else: # Not a dictionary - custom range of dates 'r' or 'range'.
		ALIAS=args[0][2]
		START_DATE,END_DATE = args[0][0], args[0][1]

	if DEBUG:
		print(YELLOW,"\nSTART_DATE: ", START_DATE, 'RELATIVE: ' , dt,'ALIAS: ', ALIAS, NO, "\n")

	df=df.assign(alias='Place Holder')
	for myalias,mylift in lifts.items():
		df['alias'] = np.where((df[NAME] == mylift), myalias, df.alias)

	DATA=df.loc[(df.alias.str.contains(ALIAS)) & (df[DATE] <= END_DATE) & (df[DATE] >= START_DATE) ].loc[:,[DATE,'record','alias',NAME]]
	ALL_DATES=df.loc[(df.alias.str.contains(ALIAS))].loc[:,[DATE,'record','alias',NAME]]
	if DATA.empty:
		print(RED+"Data Frame is empty, check variables in 'dataFrame=df.loc...'"+NO); exit()

	# Add columns and populate with data, you can also use lambada
	DATA = DATA.assign(average=round(DATA.record.mean()))
	DATA = DATA.assign(great=round(avg_dev(DATA.record))+round(DATA.record.mean()))
	DATA = DATA.assign(poor=round(DATA.record.mean())-round(avg_dev(DATA.record)))
	DATA = DATA.assign(all_average=round(ALL_DATES.record.mean()))
	DATA = DATA.assign(all_great=round(avg_dev(ALL_DATES.record))+round(ALL_DATES.record.mean()))
	DATA = DATA.assign(all_poor=round(ALL_DATES.record.mean())-round(avg_dev(ALL_DATES.record)))
	DATA = DATA.assign(apr=round(ALL_DATES.record.max())) #all time personal record, don't know if I need it.
	return (DATA)

def CHART(DATA):
	"""CHART DATA"""
	fig, ax = plt.subplots(figsize=(10,5))

	date=DATA[DATE];  record=DATA.record ; poor_color='red' ; avg_color = 'grey' ; great_color='green'
	# Relative status in current range.
	average=DATA.average ; poor=DATA.poor ; great=DATA.great; A='Average';P='Poor';G='Great' 
	if not RANGE_STATS:
		average=DATA.all_average ; poor=DATA.all_poor ; great=DATA.all_great; A='All Average'; P='All Poor'; G='All Great'#Uncomment to see above/below average all time.

	string_dates=date.min().strftime('  %b-%d-%Y')+date.max().strftime(' - %b-%d-%Y')
	ax.set_title(DATA.ename[:1].to_string(index=False)+string_dates,fontsize=16); #Not portable to say .ename, [:1] means get the first element in series

	ax.plot(date,record,color='blue',markersize='5',marker='o',markerfacecolor='black',linestyle='-',markeredgewidth='1', markeredgecolor='red',label='Daily PR')

	if STATS:
		for dashed in (('poor',poor_color),('average',avg_color),('great',great_color)):
			stat = dashed[0] ;  color = dashed[1] ; stat_data = eval(stat)
			ax.plot(
					date,stat_data,color=color,markersize='1',markevery=[0],marker='o',
					markerfacecolor='black',linestyle='--',markeredgewidth='1',
					markeredgecolor='black',label=stat,alpha=0.25)

	for line in (0,1,2,3): # label all the (x,y)
		if line > 0 and not STATS: break;
		stats = (record,great,average,poor)[line]
		for x,y in zip(date,stats):
			label = "{:.0f}".format(y)
			plt.annotate(label, # this is the text
						 (x,y), # this is the point to label
						 textcoords="offset points", # how to position the text
						 xytext=(0,10), # distance from text to points (x,y)
						 rotation=30,
						 va='center') # horizontal alignment can be left, right or center
			if line > 0: break

	ax.set_xlabel("Years",fontsize=14,color='b');
	ax.set_ylabel("Daily PR",fontsize=14,color='b');
	months_offset=pd.DateOffset(months=1)
	years_offset=pd.DateOffset(years=1)
	majorX_ticks = pd.date_range(min(date)-years_offset, max(date)+years_offset, periods=None,freq='YS',inclusive='right')
	minorX_ticks = pd.date_range(min(date)-years_offset, max(date)+years_offset, periods=None, freq='MS')

	#These are default values for greater than a year, if statement catches anything lower than 365
	ax.xaxis.set_major_formatter(mdates.DateFormatter('%b-%Y'))
	ax.xaxis.set_minor_formatter(mdates.DateFormatter('%b-%d'))

	#If less than a year, use these ticks
	if min(date) > pd.to_datetime(datetime.date.today()-relativedelta(years=1)):
		majorX_ticks = pd.date_range(min(date)-months_offset, max(date)+months_offset, periods=None,freq='MS',inclusive='right')
		minorX_ticks = pd.date_range(min(date)-months_offset, max(date)+months_offset, periods=None,freq='W-MON')
		ax.xaxis.set_major_formatter(mdates.DateFormatter('%b-%d'))
		ax.xaxis.set_minor_formatter(mdates.DateFormatter('%d'))
		ax.set_xlabel("Months",fontsize=14,color='b')

	print("MIN:", min(date), 'MAX:' , max(date) , '\nMajor X ticks:\n',majorX_ticks, '\nLENGTH: ', len(majorX_ticks))

	#ax.set_xticks(date) #uncomment if you want to have them on PR days
	ax.set_xticks(majorX_ticks)
	ax.set_xticks(minorX_ticks,minor=True)
	ax.set_ylim(min(record)-20, max(record)+20) #Chart looks better when starts at min-10 and max+10

	print(RED, max(record), min(record),NO)

	# Formatting
	ax.tick_params(axis="x", which="major", rotation=90,direction='out', width=3)
	ax.tick_params(axis="x", which="minor", rotation=90,direction="in",pad=-20, width=1,labelsize=5)
	plt.grid(which='major', color='black', linestyle='-', alpha=0.5)
	plt.grid(which='minor', color='grey', linestyle='dashed', alpha=0.2)

	# BELOW AVG = GREY, ABOVE AVG & 
	ax.fill_between(date,record,great ,label=G,where=(record > great), interpolate=True,color='g', alpha=0.25)
	ax.fill_between(date,record,poor ,label=A,where=(record >= average ) & (record < great), interpolate=True,color=avg_color, alpha=0.25)
	ax.fill_between(date,record,poor ,label=P,where=(record <= average), interpolate=True,color='r', alpha=0.25)

	ax.legend(loc='best') #Needs to be down here to show all the plots
	fig.tight_layout()#Get bottom and side to show
	if SHOW_CHART:
		plt.show(block=True)

if __name__ == "__main__":
  main()
  connect.close()
# vim: noexpandtab
